import {BaseEntity, Column, Entity, OneToMany, PrimaryGeneratedColumn} from "typeorm";
import {FoodProductIngredient} from "./foodProductIngredient.entity";

/**
 * Food product entity.
 */
@Entity("food_products")
export class FoodProduct extends BaseEntity {
	@PrimaryGeneratedColumn()
	id: number;

	@Column({
		nullable: false,
	})
	name?: string;

	/**
	 * List of ingredients of the food product.
	 */
	@OneToMany(() => FoodProductIngredient, (ingredient) => ingredient.foodProduct, {
		cascade: true,
	})
	ingredients?: FoodProductIngredient[];

	/**
	 * Computed carbon footprint of the product.
	 * Set to null if the carbon footprint couldn't be computed (e.g. because an ingredient is unknown).
	 */
	@Column({
		type: "float",
		nullable: true,
	})
	footprint: number|null = null;


	constructor(props?: {
		name: string,
		ingredients: {
			name: string,
			quantity: number,
			unit: string,
		}[],
	})
	{
		super();

		this.name = props?.name;
		this.ingredients = props?.ingredients?.map((ingredientProps) => new FoodProductIngredient(ingredientProps));
	}


	/**
	 * Try to compute (and set) the carbon footprint of the food product from its ingredients.
	 * If the carbon footprint cannot be computed, it's set to NULL.
	 * @return The computed carbon footprint.
	 */
	computeFootprint(): number|null
	{
		let totalFootprint: number = 0;
		if (this.ingredients) for (const ingredient of this.ingredients) {
			// Compute each ingredient footprint and add it to the total footprint.
			const ingredientFootprint = ingredient.computeFootprint();
			if (ingredientFootprint === null) return this.footprint = null;
			totalFootprint += ingredientFootprint;
		}
		return this.footprint = totalFootprint;
	}
}
