import {InjectRepository} from "@nestjs/typeorm";
import {Repository} from "typeorm";
import {FoodProduct} from "./foodProduct.entity";
import {CreateFoodProductDto} from "./dto/create-foodProduct.dto";
import {CarbonEmissionFactor} from "../carbonEmissionFactor/carbonEmissionFactor.entity";

export class FoodProductService
{
	constructor(
		@InjectRepository(FoodProduct) private foodProductRepository: Repository<FoodProduct>,
		@InjectRepository(CarbonEmissionFactor) private carbonEmissionFactorRepository: Repository<CarbonEmissionFactor>,
	) {}

	/**
	 * Find all created food products, with their foodprint.
	 */
	findAll(): Promise<FoodProduct[]>
	{
		return this.foodProductRepository.find();
	}

	/**
	 * Initialize a new food product object from a provided food product creation object.
	 * @param rawFoodProduct Food product creation object.
	 */
	async initialize(rawFoodProduct: CreateFoodProductDto): Promise<FoodProduct>
	{
		// Try to find all matching ingredients emission factors.
		const ingredientsEmissionFactors = await this.carbonEmissionFactorRepository.find({
			where: rawFoodProduct.ingredients.map((ingredient) => (
				{ name: ingredient.name, unit: ingredient.unit }
			)),
		});

		// Convert a simple map to retrieve ingredients emission factors from their "name + unit" key.
		const ingredientsEmissionFactorsMap = Object.fromEntries(
			ingredientsEmissionFactors.map((ingredient) => [ingredient.name + ";" + ingredient.unit, ingredient])
		);

		// Create the food product object with its ingredients.
		const foodProduct = new FoodProduct(rawFoodProduct);
		foodProduct.ingredients?.forEach((ingredient) => {
			ingredient.carbonEmissionFactor = ingredientsEmissionFactorsMap[ingredient.name + ";" + ingredient.unit] ?? null;
		});
		return foodProduct;
	}

	/**
	 * Create a new food product object from a provided food product creation object.
	 * @param rawFoodProduct Food product creation object.
	 */
	async create(rawFoodProduct: CreateFoodProductDto): Promise<FoodProduct>
	{ // Initialize and save the food product.
		return (await this.initialize(rawFoodProduct)).save();
	}
}
