import {BaseEntity, Column, Entity, ManyToOne, PrimaryGeneratedColumn} from "typeorm";
import {FoodProduct} from "./foodProduct.entity";
import {CarbonEmissionFactor} from "../carbonEmissionFactor/carbonEmissionFactor.entity";

/**
 * Food product entity.
 */
@Entity("food_products_ingredients")
export class FoodProductIngredient extends BaseEntity {
	@PrimaryGeneratedColumn()
	id: number;

	/**
	 * The food product of the ingredient.
	 */
	@ManyToOne(() => FoodProduct, (product) => product.ingredients, {
		nullable: false,
	})
	foodProduct: FoodProduct;

	/**
	 * The carbon emission factor of the ingredient.
	 */
	@ManyToOne(() => CarbonEmissionFactor, {
		nullable: true,
	})
	carbonEmissionFactor: CarbonEmissionFactor|null = null;

	@Column({
		nullable: false,
	})
	name?: string;

	@Column({
		type: "float",
		nullable: false,
	})
	quantity?: number;

	@Column({
		nullable: false,
	})
	unit?: string;

	constructor(props?: {
		name: string,
		quantity: number,
		unit: string,
	})
	{
		super();

		this.name = props?.name;
		this.quantity = props?.quantity;
		this.unit = props?.unit;
	}

	/**
	 * Try to compute the carbon footprint of the food product ingredient.
	 * If the carbon footprint cannot be computed, it's set to NULL.
	 * @return The computed carbon footprint.
	 */
	computeFootprint(): number|null
	{
		if (this.carbonEmissionFactor)
			return (this.quantity ?? 0) * this.carbonEmissionFactor.emissionCO2eInKgPerUnit;
		else
			return null;
	}
}
